"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createComplexObjectBuilder = void 0;
const PropertyType_1 = require("../../models/property/PropertyType");
const FormBuilder_1 = require("../../components/FormBuilder");
const DynamicArrayField_1 = require("./components/DynamicArrayField");
const react_1 = __importDefault(require("react"));
const __1 = require("../..");
const BuildPropertyRenderProps_1 = require("../helpers/BuildPropertyRenderProps");
const createComplexObjectBuilder = (labelRender, validationMessage, loadingSpinner) => {
    const id = "internal_complexbuilder";
    const DefaultLabelRender = (props) => react_1.default.createElement("label", { key: props.parentKey + "-label" }, props.propertySchema.displayName);
    const defaultValidationMessageElement = (message) => react_1.default.createElement("p", null, message);
    const defaultSpinner = (props, context) => react_1.default.createElement("p", null, "Loading...");
    const BuilderLabelRender = labelRender !== null && labelRender !== void 0 ? labelRender : DefaultLabelRender;
    const builderValidationMessageElement = validationMessage !== null && validationMessage !== void 0 ? validationMessage : defaultValidationMessageElement;
    const loadingSpinnerComponent = () => loadingSpinner !== null && loadingSpinner !== void 0 ? loadingSpinner : defaultSpinner;
    const convertPropertyOverrides = (propertyOverrides, property) => {
        if (propertyOverrides == undefined)
            return undefined;
        let clone = Object.assign({}, propertyOverrides);
        if (propertyOverrides.disabledProps != null) {
            if (Array.isArray(propertyOverrides.disabledProps)) {
                if (propertyOverrides.disabledProps.indexOf(property) >= 0) {
                    clone.disabledProps = true;
                }
            }
            else {
                clone.disabledProps = !!propertyOverrides.disabledProps;
            }
        }
        console.log(clone);
        return clone;
    };
    const build = (renderProps, property, schema) => {
        let { item, validationResults } = renderProps;
        if (item == null)
            throw Error("item is null");
        if (schema == null)
            throw Error("schema is null");
        let info = (0, BuildPropertyRenderProps_1.buildPropertyRenderInfo)(renderProps, schema, property);
        let validationMark = (0, __1.getValidationMarkForProperty)(renderProps, property);
        const WrapInLabel = (element, addErrormessage) => {
            return (react_1.default.createElement("div", { className: "formbuilder-property", key: info.props.key },
                react_1.default.createElement(BuilderLabelRender, { key: `${info.key}-labelcontainer`, propertySchema: schema, hideLabel: info.props.options.hideLabel, parentKey: info.key, validationMark: validationMark }),
                element,
                (addErrormessage && info.props.errorMessage) && builderValidationMessageElement(info.props.errorMessage)));
        };
        const propertyType = PropertyType_1.propertyTypes;
        switch (schema.propertyType) {
            case propertyType.array:
                return { found: true, element: WrapInLabel(react_1.default.createElement(DynamicArrayField_1.DynamicArrayField, Object.assign({}, schema, info.props))) };
            case propertyType.custom:
                let customConfig = schema.config;
                if (!(customConfig === null || customConfig === void 0 ? void 0 : customConfig.component))
                    return { found: false, element: undefined };
                // const Comp = this.customComponents[customConfig?.componentName];
                const Comp = customConfig.component;
                return { found: true, element: react_1.default.createElement(react_1.default.Fragment, null, WrapInLabel(react_1.default.createElement(Comp, Object.assign({ key: info.key }, info.props, schema, customConfig.componentProps)))) };
            case propertyType.formItem:
                let formConfig = schema.config;
                if (!(formConfig === null || formConfig === void 0 ? void 0 : formConfig.schemaConfig))
                    return { found: false, element: undefined };
                let formItemProps = {
                    item: item[property],
                    schemaConfig: formConfig.schemaConfig,
                    groupContainer: formConfig.groupContainer,
                    groupRender: formConfig.groupRender,
                    onPropertyChange: (item, prop, value) => info.props.onChange(item),
                    validationResult: validationResults,
                    validationResultPrefix: property,
                    propertyOverrides: convertPropertyOverrides(renderProps.propertyOverrides, property)
                };
                // handle dynamic schema coming from formitem config
                if (formConfig.schemaConfig.dynamicSchema != null) {
                    const dynamicSchema = formConfig.schemaConfig = formConfig.schemaConfig.dynamicSchema(item);
                    formConfig.schemaConfig = dynamicSchema;
                    formItemProps.schemaConfig = dynamicSchema;
                }
                let addErrormessage = false;
                // used for having properties as array of formitem
                let listProps = schema === null || schema === void 0 ? void 0 : schema.listProps;
                if (listProps != undefined) {
                    formItemProps.onPropertyChange = undefined;
                    listProps.config.onItemsChange = items => info.props.onChange(items);
                    addErrormessage = true;
                }
                // types sucks sometimes!! (ewi)
                return { found: true, element: react_1.default.createElement(react_1.default.Fragment, null, WrapInLabel(react_1.default.createElement(FormBuilder_1.FormBuilder, Object.assign({ key: info.key, keyPrefix: info.key }, formItemProps, formConfig, { listProps: listProps })), addErrormessage)) };
            default: return { found: false, element: undefined };
        }
    };
    return { id, build, loadingComponent: loadingSpinnerComponent };
};
exports.createComplexObjectBuilder = createComplexObjectBuilder;
//# sourceMappingURL=ComplexObjectBuilder.js.map