import React from 'react';
import { IFormItemOptions } from '../interfaces/options/IFormItemOptions';
import { IFormSchema } from '../interfaces/schema/IFormSchema';
import { IValidationResult } from '../models/validation/IValidationResult';
import { ValidationEventType } from '../models/validation/ValidationEventType';
import { ValidationOverride } from '../models/validation/ValidationOverride';
import { ValidationResult } from '../models/validation/ValidationResult';
import { IDynamicSchemaConfig } from './config/IFormBuilderListConfig';
import { IFormGrouping } from '../interfaces/IFormGrouping';
import { IPropertyOverrides } from '../interfaces/IPropertyOverrides';
import { ILoadingProps } from '../builders/interfaces/ILoadingProps';
import { RequireOnlyOne } from '../interfaces/types/Partials';
import { ISchemaConfig } from './FormBuilder';
import { IFormItem } from '../interfaces/form/IFormItem';
export interface IFormItemProps<T extends IFormItem> {
    schemaConfig: RequireOnlyOne<ISchemaConfig<T>, "registeredSchemaKey" | "schemaProvider">;
    item: T;
    schema?: IFormSchema<T>;
    dynamicSchema?: IDynamicSchemaConfig<T>;
    onPropertyChange?: (item: T, prop: string, value: any) => void;
    groupContainer?: React.FC<{
        groupings: Array<IFormGrouping>;
    }>;
    groupRender?: React.FC<{
        grouping: IFormGrouping;
    }>;
    validationOverride?: ValidationOverride;
    formItemConfigOverrides?: Partial<IFormItemOptions<T>>;
    propertyOverrides?: IPropertyOverrides<T>;
    loadingProps?: ILoadingProps;
    /*** DO NOT USE! only used internally */
    validationResult?: IValidationResult;
    /*** DO NOT USE! only used internally */
    validationResultPrefix?: string;
    keyPrefix?: string;
}
export declare type FormRef<T extends IFormItem> = {
    getItem: () => T;
    validateItem: (type?: ValidationEventType) => Promise<ValidationResult>;
};
export declare const Form: React.ForwardRefExoticComponent<IFormItemProps<IFormItem> & React.RefAttributes<unknown>>;
