import React, { Ref } from 'react';
import { IFormItemOptions } from '../interfaces/options/IFormItemOptions';
import { IFormSchema } from '../interfaces/schema/IFormSchema';
import { ValidationOverride } from '../models/validation/ValidationOverride';
import { ValidationResult } from '../models/validation/ValidationResult';
import { IFormBuilderListConfig, IFormBuilderListSearchConfig, IFormBuilderListColumns, IFormBuilderListMenuConfig, IFormBuilderListEditorConfig, IFormListShimmerConfig } from './config/IFormBuilderListConfig';
import { IFormGrouping } from '../interfaces/IFormGrouping';
import { IPropertyOverrides } from '../interfaces/IPropertyOverrides';
import { ILoadingProps } from '../builders/interfaces/ILoadingProps';
import { ISchemaProvider } from '../interfaces/schema/ISchemaProvider';
import { RequireOnlyOne, RecursivePartial } from '..';
import { IFormItem } from '../interfaces/form/IFormItem';
export declare type FormBuilderItemType<T extends IFormItem> = T | Array<T>;
export interface FormBuilderSingleItemProps<T extends IFormItem> {
    onPropertyChange?: (item: T, prop: string, value: any) => void;
}
export interface IFormBuilderListProps<T extends IFormItem> {
    config: IFormBuilderListConfig<T>;
    onItemChange?: (item: T) => void;
    searchConfig?: IFormBuilderListSearchConfig;
    columnConfig?: IFormBuilderListColumns<T>;
    menuConfig?: IFormBuilderListMenuConfig<T>;
    editorConfig?: IFormBuilderListEditorConfig<T>;
    shimmerConfig?: IFormListShimmerConfig;
}
export interface ISchemaConfig<T extends IFormItem> {
    registeredSchemaKey?: string;
    schemaProvider?: ISchemaProvider<T>;
}
export interface IFormBuilderProps<T extends IFormItem> {
    item: FormBuilderItemType<T>;
    schemaConfig: RequireOnlyOne<ISchemaConfig<T>, "registeredSchemaKey" | "schemaProvider">;
    singleItemProps?: FormBuilderSingleItemProps<T>;
    listProps?: IFormBuilderListProps<T>;
    groupContainer?: React.FC<{
        groupings: Array<IFormGrouping>;
    }>;
    groupRender?: React.FC<{
        grouping: IFormGrouping;
    }>;
    validationOverride?: ValidationOverride;
    formItemConfigOverrides?: RecursivePartial<IFormItemOptions<T>>;
    propertyOverrides?: IPropertyOverrides<T>;
    loadingProps?: ILoadingProps;
    keyPrefix?: string;
    overrideSchema?: IFormSchema<T>;
}
export declare type FormBuilderRef<T extends IFormItem> = {
    getItem: () => FormBuilderItemType<T>;
    validateItem: () => Promise<ValidationResult>;
};
export declare const FormBuilder: <T extends IFormItem>(props: IFormBuilderProps<T> & {
    formRef?: React.Ref<any> | undefined;
}) => JSX.Element | null;
