"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FormList = void 0;
const react_1 = __importStar(require("react"));
const FormBuilderInitializer_1 = require("../utils/FormBuilderInitializer");
const useStateRef_1 = require("../hooks/useStateRef");
const FormListHelper_1 = require("../builders/helpers/FormListHelper");
exports.FormList = (0, react_1.forwardRef)((props, ref) => {
    var _a, _b, _c, _d, _e;
    // verify that the formbuilder is set up correctly
    FormBuilderInitializer_1.formbuilder.verify();
    const [listProps, setListProps] = (0, react_1.useState)(Object.assign({}, props));
    const [items, setItems, itemsRef] = (0, useStateRef_1.useStateRef)((_a = props.items) !== null && _a !== void 0 ? _a : []);
    const [filteredItems, setFilteredItems, filteredItemsRef] = (0, useStateRef_1.useStateRef)((_b = props.items) !== null && _b !== void 0 ? _b : []);
    const [selectedItems, setSelectedItems, selectedItemsRef] = (0, useStateRef_1.useStateRef)([]);
    const [columns, setColumns, columnRef] = (0, useStateRef_1.useStateRef)();
    const [enableShimmer, setEnableShimmer] = (0, react_1.useState)(false);
    (0, react_1.useEffect)(() => {
        var _a, _b, _c, _d;
        setItems((_a = props.items) !== null && _a !== void 0 ? _a : []);
        setFilteredItems((_b = props.items) !== null && _b !== void 0 ? _b : []);
        // update selected items, when changes to items from outside (ewi)
        if ((_c = selectedItemsRef.current) === null || _c === void 0 ? void 0 : _c.length) {
            const changedItems = ((_d = props.items) !== null && _d !== void 0 ? _d : []).filter(_ => selectedItemsRef.current.findIndex(r => props.listProps.config.itemIdentifier(r) == props.listProps.config.itemIdentifier(_)) >= 0);
            setSelectedItems(changedItems);
        }
    }, [props.items]);
    (0, react_1.useImperativeHandle)(ref, () => ({
        getItem: () => items
    }), [items]);
    (0, react_1.useEffect)(() => {
        let clone = Object.assign({}, props);
        let callback = clone.listProps.config.onItemsChange;
        clone.listProps.config.onItemsChange = (items) => {
            setItems(items);
            if (callback)
                callback(items);
        };
        setListProps(clone);
    }, [props.listProps, props.schema]);
    (0, react_1.useEffect)(() => {
        const cols = buildColumns();
        setColumns(cols);
    }, []);
    (0, react_1.useEffect)(() => {
        var _a, _b;
        const shimmerConfig = (_a = props.listProps) === null || _a === void 0 ? void 0 : _a.shimmerConfig;
        if (shimmerConfig === null || shimmerConfig === void 0 ? void 0 : shimmerConfig.forceShimmer) {
            setEnableShimmer(true);
        }
        else if ((shimmerConfig === null || shimmerConfig === void 0 ? void 0 : shimmerConfig.autoShimmerOnEmptyList) && !!!(items === null || items === void 0 ? void 0 : items.length)) {
            setEnableShimmer(true);
            setTimeout(() => setEnableShimmer(false), (_b = shimmerConfig === null || shimmerConfig === void 0 ? void 0 : shimmerConfig.autoShimmerTimeout) !== null && _b !== void 0 ? _b : 5000);
        }
        else {
            setEnableShimmer(false);
        }
    }, [items, (_c = props.listProps) === null || _c === void 0 ? void 0 : _c.shimmerConfig, (_e = (_d = props.listProps) === null || _d === void 0 ? void 0 : _d.shimmerConfig) === null || _e === void 0 ? void 0 : _e.forceShimmer]);
    if ((listProps === null || listProps === void 0 ? void 0 : listProps.schema) == null)
        return null;
    const itemChangeInCollection = (collection, item) => {
        let clone = [...collection];
        let itemIndex = clone.findIndex(_ => props.listProps.config.itemIdentifier(_) == props.listProps.config.itemIdentifier(item));
        if (itemIndex < 0)
            clone.unshift(item);
        else
            clone[itemIndex] = Object.assign({}, item);
        return clone;
    };
    const onItemChange = (item) => {
        if (item == null)
            return;
        // items
        let changedItems = itemChangeInCollection(itemsRef.current, item);
        setItems(changedItems);
        // filtered
        let changedFilteredItems = itemChangeInCollection(filteredItemsRef.current, item);
        setFilteredItems(changedFilteredItems);
        // selected
        let changedSelectedItems = itemChangeInCollection(selectedItemsRef.current, item);
        setSelectedItems(changedSelectedItems);
        if (props.listProps.onItemChange)
            props.listProps.onItemChange(Object.assign({}, item));
        if (props.listProps.config.onItemsChange)
            props.listProps.config.onItemsChange([...changedItems]);
    };
    const columnValueRender = (propInfo) => {
        var _a;
        let valueRender = (_a = propInfo === null || propInfo === void 0 ? void 0 : propInfo.listItemOptions) === null || _a === void 0 ? void 0 : _a.customValueRender;
        if (valueRender == undefined)
            return undefined;
        else {
            return (item) => valueRender(item, ((item) => onItemChange(item)));
        }
    };
    const deleteItems = (pre) => {
        let sItems = [...selectedItems];
        // pre event (can cancel the operation)
        if (pre) {
            let result = pre(sItems);
            if (result === false)
                return;
        }
        let itemsLeft = [];
        let filteredItemsLeft = [];
        // not all items
        if (items.length != sItems.length) {
            itemsLeft = items.filter(_ => sItems.indexOf(_) < 0);
            filteredItemsLeft = filteredItems.filter(_ => sItems.indexOf(_) < 0);
        }
        setItems(itemsLeft);
        setFilteredItems(filteredItemsLeft);
        setSelectedItems([]);
        if (props.listProps.config.onItemsChange)
            props.listProps.config.onItemsChange([...itemsLeft]);
    };
    const buildColumns = () => {
        var _a, _b;
        let propertyOptions = props.schema.options.properties;
        let columnOptions = (_a = props.listProps.columnConfig) === null || _a === void 0 ? void 0 : _a.columnsPicks;
        let propKeys = Object.keys(propertyOptions);
        let filteredPropKeys = FormListHelper_1.formListHelper.filterColumns(propKeys, columnOptions);
        let columns = filteredPropKeys.map(_ => {
            var _a, _b;
            let propInfo = propertyOptions[_];
            let col = {
                key: _,
                fieldName: _,
                name: propInfo.displayName,
                onRender: columnValueRender(propInfo),
                minWidth: (_a = propInfo.listItemOptions) === null || _a === void 0 ? void 0 : _a.minWidth,
                isResizable: (_b = propInfo.listItemOptions) === null || _b === void 0 ? void 0 : _b.isResizable
            };
            return col;
        });
        let sortOrder = (_b = props.listProps.columnConfig) === null || _b === void 0 ? void 0 : _b.columnOrder;
        if (sortOrder != null)
            return columns.sort((a, b) => {
                if (sortOrder.indexOf(a.fieldName) == -1)
                    return 1;
                if (sortOrder.indexOf(b.fieldName) == -1)
                    return -1;
                return sortOrder.indexOf(a.fieldName) - sortOrder.indexOf(b.fieldName);
            });
        else
            return columns;
    };
    const sortColumn = (column) => {
        if (column == null)
            return;
        const newColumns = columns.slice();
        const currColumn = columnRef.current.filter(currCol => column.key === currCol.key)[0];
        newColumns.forEach((newCol) => {
            if (newCol === currColumn) {
                currColumn.isSortedDescending = !currColumn.isSortedDescending;
                currColumn.isSorted = true;
            }
            else {
                newCol.isSorted = false;
                newCol.isSortedDescending = true;
            }
        });
        const property = currColumn.fieldName;
        const newItems = FormListHelper_1.formListHelper.copyAndSort(filteredItemsRef.current, property, currColumn.isSortedDescending);
        setColumns(newColumns);
        setFilteredItems(newItems);
    };
    const ListContainer = FormBuilderInitializer_1.formbuilder.formItemRender.list(); // WTF!! added readOnly to listprops and suddently it wont compile - works fine though (ewi)
    return (react_1.default.createElement("div", { className: "formbuilder-listcontainer", key: `${props.keyPrefix}` },
        react_1.default.createElement(ListContainer, Object.assign({}, listProps, { items: items, filteredItems: filteredItems, selectedItems: selectedItems, onItemChange: onItemChange !== null && onItemChange !== void 0 ? onItemChange : ((item) => { }), deleteItems: deleteItems, columnValueRender: columnValueRender, columns: columns !== null && columns !== void 0 ? columns : [], updateItems: setItems, updateFilteredItems: setFilteredItems, updateSelectedItems: setSelectedItems, sortColumn: sortColumn, readOnly: props.readOnly, shimmer: enableShimmer }))));
});
//# sourceMappingURL=FormList.js.map