/// <reference types="react" />
import { IFormItem } from "../../interfaces/form/IFormItem";
import { RequireOnlyOne } from "../../interfaces/types/Partials";
import { ValidationResult } from "../../models/validation/ValidationResult";
import { ISchemaConfig } from "../FormBuilder";
export interface IFormBuilderListConfig<T extends IFormItem> {
    itemIdentifier: (item: T) => string;
    onItemsChange?: (items: Array<T>) => void;
    multiSelect?: boolean;
    disableItemInvoke?: boolean;
}
export declare enum FormListColumnsPickType {
    Only = 0,
    Without = 1
}
export interface IFormListColumnsPicks<T> {
    columns: Array<keyof T>;
    pickType: FormListColumnsPickType;
}
export interface IFormListShimmerConfig {
    /** amount of lines used by the shimmer - defaults to 10 */
    shimmerLines?: number;
    /** will force the list to shimmer until set to false again */
    forceShimmer?: boolean;
    /** will enable shimmer automatically when item array is empty */
    autoShimmerOnEmptyList?: boolean;
    /** the timeout value in miliseconds for automatic shimmer - to prevent eternal shimmering (defaults to 5000) */
    autoShimmerTimeout?: number;
    /** element to render if no items are present */
    noItemsElement?: JSX.Element;
}
export interface IFormBuilderListColumns<T> {
    columnOrder?: Array<keyof T>;
    columnsPicks?: IFormListColumnsPicks<T>;
    resizableColumns?: Array<keyof T> | true;
    columnWidths?: Array<{
        column: string;
        width: number;
    }> | number;
}
export declare enum IFormBuilderListMenuItemSelectionMode {
    None = 1,
    Single = 2,
    Multi = 4
}
export interface IFormBuilderListMenuItem<T extends IFormItem> {
    title: string;
    selectionMode: IFormBuilderListMenuItemSelectionMode;
    action: (items: Array<T>) => boolean | void;
    subMenuItems?: Array<IFormBuilderListMenuItem<T>>;
    disabled?: (selectedItems: Array<T>) => boolean;
    iconName?: string;
}
export interface IFormBuilderListMenuConfig<T extends IFormItem> {
    actions?: (newItem: (pre?: (item: T) => void | boolean) => void, editItem: (pre?: (item: T) => void | boolean) => void, deleteItems: (pre?: (items: Array<T>) => void | boolean) => void) => Array<IFormBuilderListMenuItem<T>>;
}
export interface IFormBuilderListSearchConfig {
    searchEnabled?: boolean;
    searchableFields: Array<string>;
    searchPlaceHolder?: string;
}
export interface IDynamicSchemaConfig<T> extends IFormItem {
    dynamicKey: (item: T) => string;
    schemaConfig?: RequireOnlyOne<ISchemaConfig<T>, "registeredSchemaKey" | "schemaProvider">;
    useEmptyItem?: boolean;
}
export declare enum FormBuilderListEditorType {
    Dialog = 0,
    Panel = 1
}
export interface IFormBuilderListEditorConfig<T> {
    title: (item: T, newItemMode?: boolean) => string;
    description?: string;
    type: FormBuilderListEditorType;
    dynamicForm?: IDynamicSchemaConfig<T>;
    customFooter?: (save: () => Promise<void>, dismiss: () => Promise<void>, validate: () => Promise<ValidationResult | undefined>, getItem: () => T | any) => JSX.Element;
}
