"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formValidator = void 0;
const FormSchemaUtil_1 = require("../../utils/common/FormSchemaUtil");
const validateRule = (item, property, rule, onEvent, results, validationResultPrefix) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b;
    if (rule == null)
        return;
    let propertyIndexer = validationResultPrefix ? `${validationResultPrefix}.${property}` : property;
    if (Array.isArray(rule)) {
        rule.forEach(_ => {
            var _a;
            let validated = (onEvent != undefined && !((_.validateOn & onEvent) === onEvent)) || (!_.disabled && ((_a = _.validationRule) === null || _a === void 0 ? void 0 : _a.call(_, item)));
            if (results[propertyIndexer] == null)
                results[propertyIndexer] = {};
            results[propertyIndexer][_.id] = { message: validated ? "" : _.validationMessage, success: !!validated };
        });
    }
    else {
        if (rule.nestedValidation != undefined) {
            let childSchema = rule.nestedValidation.schemaKey
                ? yield FormSchemaUtil_1.formSchemaUtil.getSchemaFromMap(rule.nestedValidation.schemaKey)
                : yield ((_a = rule.nestedValidation.schemaProvider) === null || _a === void 0 ? void 0 : _a.getSchema());
            if (item[property] != null) {
                results = yield exports.formValidator.validate(item[property], undefined, childSchema, onEvent, results, property); // TODO: maybe set all events here ('undefined'), to prevent validation messages to dissapear when other properties change (ewi)
            }
        }
        else {
            let validated = (onEvent != undefined && !((rule.validateOn & onEvent) === onEvent)) || (!rule.disabled && ((_b = rule.validationRule) === null || _b === void 0 ? void 0 : _b.call(rule, item)));
            if (results[propertyIndexer] == null)
                results[propertyIndexer] = {};
            results[propertyIndexer][rule.id] = { message: validated ? "" : rule.validationMessage, success: !!validated };
        }
    }
});
exports.formValidator = {
    // TODO: handle nested validation on complex objects, more smooth (ewi)
    // TODO: handle validation differently than using 'onEvent'?! (ewi)
    validate: (item, prop, schema, onEvent, existingResults, validationResultPrefix) => __awaiter(void 0, void 0, void 0, function* () {
        var _c;
        let validation = (_c = schema === null || schema === void 0 ? void 0 : schema.options) === null || _c === void 0 ? void 0 : _c.validation;
        if (validation == null)
            return existingResults;
        if (validation.ignoreValidation)
            return existingResults;
        let rules = validation.validationRules;
        if (rules == null)
            return existingResults;
        let results = existingResults !== null && existingResults !== void 0 ? existingResults : {};
        // specific prop needs validation
        if (prop !== undefined) {
            let rule = rules[prop];
            yield validateRule(item, prop, rule, onEvent, results, validationResultPrefix);
        }
        else {
            for (const key of Object.keys(rules)) {
                let rule = rules[key];
                yield validateRule(item, key, rule, onEvent, results, validationResultPrefix);
            }
        }
        return results;
    })
};
//# sourceMappingURL=FormValidator.js.map