"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useStateRef = void 0;
const react_1 = require("react");
const isFunction = (setStateAction) => typeof setStateAction === "function";
// type UseStateRef = {
//     <S>(initialState: S | (() => S)): [S, Dispatch<SetStateAction<S>>, ReadOnlyRefObject<S>];
//     <S = undefined>(): [S | undefined, Dispatch<SetStateAction<S | undefined>>, ReadOnlyRefObject<S | undefined>];
// };
/**
 * Hook for adding a ref to the regular usestate hook.
 * The ref is automatically updated when setstate is called.
 * @returns a regular usestate where a ref is included
 */
const useStateRef = (initialState) => {
    const [state, setState] = (0, react_1.useState)(initialState); // sometimes types are fucked in TS (ewi) // TODO!!
    const ref = (0, react_1.useRef)(state);
    const dispatch = (0, react_1.useCallback)((setStateAction) => {
        ref.current = isFunction(setStateAction) ? setStateAction(ref.current) : setStateAction;
        setState(ref.current);
    }, []);
    return [state, dispatch, ref];
};
exports.useStateRef = useStateRef;
//# sourceMappingURL=useStateRef.js.map