/// <reference types="react" />
import { IFormItemRender } from "../components/helpers/FormItemRender";
import { IFormItem } from "../interfaces/form/IFormItem";
import { ISchemaProvider } from "../interfaces/schema/ISchemaProvider";
import { RecursivePartial } from "../interfaces/types/Partials";
import { IFormbuilderLanguage } from "../models/language/ILanguage";
import { FormLabel, IFormItemBuilder } from "../builders/interfaces/IFormItemBuilder";
import { ILoadingProps } from "../builders/interfaces/ILoadingProps";
export interface IFormBuilder {
    formItemRender: IFormItemRender;
    /** initialize the formbuilder */
    initialize: () => IFormBuilder;
    /** use the complex builder, which is used for nested forms or render with custom components */
    usingComplexBuilder: (labelRender?: FormLabel, validationMsgElement?: (message: string) => JSX.Element, loadingSpinner?: React.ElementType<ILoadingProps>) => IFormBuilder;
    /** Add one or more of the standard builders or your own custom ones */
    withBuilders: (...builders: Array<IFormItemBuilder>) => IFormBuilder;
    /** override all or parts of the text used in the formbuilder */
    withLanguage: (language: RecursivePartial<IFormbuilderLanguage>) => IFormBuilder;
    /** use this to be able to register schemas, making the implementation of child forms prettier */
    registerSchemaProvider: <T extends IFormItem>(provider: ISchemaProvider<T>) => void;
    /** will get the schemaprovider by it's key */
    getSchemaProvider: <T extends IFormItem>(key: string) => ISchemaProvider<T> | undefined;
    /** verifies that the formbuilder is setup correctly - throws an error if not */
    verify: () => void;
}
export declare const formbuilder: IFormBuilder;
