"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mergeSchema = exports.fetchSchema = exports.fetchSchemaFromMap = void 0;
const __1 = require("../..");
const FormSchemaUtil_1 = require("./FormSchemaUtil");
const MergeObjects_1 = require("./MergeObjects");
const fetchSchemaFromMap = (schemaMapKey, formItemConfigOverrides, propertyOverrides, dynamicKey) => __awaiter(void 0, void 0, void 0, function* () {
    let schema = yield FormSchemaUtil_1.formSchemaUtil.getSchemaFromMap(schemaMapKey, dynamicKey);
    return (0, exports.mergeSchema)(schema, formItemConfigOverrides, propertyOverrides, dynamicKey);
});
exports.fetchSchemaFromMap = fetchSchemaFromMap;
const fetchSchema = (schemaProvider, formItemConfigOverrides, propertyOverrides, dynamicKey) => __awaiter(void 0, void 0, void 0, function* () {
    let schema = yield schemaProvider.getSchema(dynamicKey);
    return (0, exports.mergeSchema)(schema, formItemConfigOverrides, propertyOverrides, dynamicKey);
});
exports.fetchSchema = fetchSchema;
const mergeSchema = (schema, formItemConfigOverrides, propertyOverrides, dynamicKey) => {
    if (schema == null)
        return undefined;
    // merge with overrides
    let merged = (0, MergeObjects_1.mergeDeep)(schema === null || schema === void 0 ? void 0 : schema.options, formItemConfigOverrides);
    // property overrides
    let disabledOverrides = propertyOverrides === null || propertyOverrides === void 0 ? void 0 : propertyOverrides.disabledProps;
    let hiddenOverrides = propertyOverrides === null || propertyOverrides === void 0 ? void 0 : propertyOverrides.hiddenProps;
    Object.keys(merged.properties).forEach(_ => {
        if (disabledOverrides != null) {
            let disable = merged.properties[_].disable;
            if (Array.isArray(disabledOverrides)) {
                if (disabledOverrides.indexOf(_) >= 0) {
                    disable = () => true;
                }
            }
            else {
                disable = () => !!disabledOverrides;
            }
            merged.properties[_].disable = disable;
        }
        if (hiddenOverrides != null) {
            const found = hiddenOverrides.columns.indexOf(_) >= 0;
            if (hiddenOverrides.pickType == __1.FormListColumnsPickType.Only) {
                merged.properties[_].hide = () => found;
            }
            else if (hiddenOverrides.pickType == __1.FormListColumnsPickType.Without) {
                merged.properties[_].hide = () => !found;
            }
        }
    });
    schema.options = merged;
    return schema;
};
exports.mergeSchema = mergeSchema;
//# sourceMappingURL=SchemaFetch.js.map